from __future__ import annotations

import logging

from sqlalchemy import Column, ForeignKey, String
from sqlalchemy.orm import Query, backref, relationship
from typing_extensions import Self

import ckan.model as model
import ckan.types as types
from ckan.plugins import toolkit as tk

import ckanext.permissions.types as perm_types

log = logging.getLogger(__name__)


class Role(tk.BaseModel):
    __tablename__ = "perm_role"

    id = Column(String, primary_key=True)
    label = Column(String, nullable=False)
    description = Column(String, nullable=False)

    @classmethod
    def create(cls, id: str, label: str, description: str) -> Self:
        role = cls(id=id, label=label, description=description)

        model.Session.add(role)
        model.Session.commit()

        return role

    @classmethod
    def get(cls, role: str) -> Self | None:
        return model.Session.query(cls).filter(cls.id == role).one_or_none()

    @classmethod
    def all(cls) -> list[Self]:
        return [role.dictize({}) for role in model.Session.query(cls).all()]

    def update(self, description: str) -> None:
        self.description = description

        model.Session.commit()

    def dictize(self, context: types.Context) -> perm_types.Role:
        return perm_types.Role(
            id=str(self.id),
            label=str(self.label),
            description=str(self.description),
        )

    def delete(self) -> None:
        model.Session.delete(self)
        model.Session.commit()


class UserRole(tk.BaseModel):
    __tablename__ = "perm_user_role"

    user_id = Column(String, ForeignKey("user.id", ondelete="CASCADE"), primary_key=True)
    role_id = Column(String, ForeignKey("perm_role.id", ondelete="CASCADE"), primary_key=True)

    scope = Column(String, primary_key=True, default="global")
    scope_id = Column(String, nullable=True)

    user = relationship(
        model.User,
        backref=backref("roles", cascade="all, delete"),
    )

    role = relationship(Role, cascade="all, delete")

    @classmethod
    def get(cls, user_id: str, scope: str = "global", scope_id: str | None = None) -> list[Self]:
        query: Query = model.Session.query(cls).filter(cls.user_id == user_id).filter(cls.scope == scope)

        if scope_id:
            query = query.filter(cls.scope_id == scope_id)

        return query.all()

    @classmethod
    def create(
        cls,
        user_id: str,
        role: str,
        scope: str = "global",
        scope_id: str | None = None,
    ) -> Self:
        for user_role in cls.get(user_id, scope, scope_id):
            if user_role.role_id != role:
                continue

            return user_role

        user_role = cls(user_id=user_id, role_id=role, scope=scope, scope_id=scope_id)

        model.Session.add(user_role)
        model.Session.commit()

        return user_role

    @classmethod
    def clear_user_roles(cls, user_id: str, scope: str = "", scope_id: str | None = None) -> None:
        perm = model.Session.query(UserRole).filter(UserRole.user_id == user_id)

        if scope:
            perm = perm.filter_by(scope=scope)
            if scope_id:
                perm = perm.filter_by(scope_id=scope_id)

        perm.delete()
        model.Session.commit()

    @classmethod
    def delete(cls, user_id: str, role: str) -> None:
        model.Session.query(cls).filter(cls.user_id == user_id, cls.role_id == role).delete()
        model.Session.commit()


class RolePermission(tk.BaseModel):
    __tablename__ = "perm_role_permission"

    role_id = Column(String, ForeignKey("perm_role.id"), primary_key=True)
    permission = Column(String, primary_key=True)

    @classmethod
    def get(cls, role_id: str, permission: str) -> Self | None:
        query: Query = model.Session.query(cls).filter(cls.role_id == role_id, cls.permission == permission)

        return query.one_or_none()

    @classmethod
    def create(cls, role_id: str, permission: str, defer_commit: bool = True) -> Self:
        role_permission = cls(role_id=role_id, permission=permission)

        model.Session.add(role_permission)

        if defer_commit:
            model.Session.commit()

        return role_permission

    def delete(self) -> None:
        model.Session().autoflush = False
        model.Session.delete(self)



"""Init tables

Revision ID: a849104ccfdc
Revises:
Create Date: 2024-02-14 17:11:12.064281

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "a849104ccfdc"
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    op.create_table(
        "perm_role",
        sa.Column("id", sa.String(), primary_key=True),
        sa.Column("label", sa.String(), nullable=False),
        sa.Column("description", sa.String(), nullable=False),
    )

    op.create_table(
        "perm_user_role",
        sa.Column(
            "user_id",
            sa.String(),
            sa.ForeignKey("user.id", ondelete="CASCADE"),
            primary_key=True,
        ),
        sa.Column(
            "role_id",
            sa.String(),
            sa.ForeignKey("perm_role.id", ondelete="CASCADE"),
            primary_key=True,
        ),
        sa.Column(
            "scope",
            sa.String(),
            primary_key=True,
            default="global",
        ),
        sa.Column(
            "scope_id",
            sa.String(),
            nullable=True,
        ),
    )

    op.create_table(
        "perm_role_permission",
        sa.Column(
            "role_id",
            sa.String(),
            sa.ForeignKey("perm_role.id", ondelete="CASCADE"),
            primary_key=True,
        ),
        sa.Column("permission", sa.String(), nullable=False, primary_key=True),
    )


def downgrade():
    op.drop_table("perm_role_permission")
    op.drop_table("perm_user_role")
    op.drop_table("perm_role")



from __future__ import annotations

from typing import cast

import ckan.plugins.toolkit as tk
from ckan import model
from ckan.logic import validate
from ckan.types import Context, DataDict

from ckanext.permissions import model as perm_model
from ckanext.permissions import types as perm_types
from ckanext.permissions import utils as perm_utils
from ckanext.permissions.logic import schema


@validate(schema.role_create)
def permission_role_create(context: Context, data_dict: DataDict) -> perm_types.Role:
    tk.check_access("manage_user_roles", context, data_dict)
    return perm_model.Role.create(**data_dict).dictize(context)


@validate(schema.role_delete)
def permission_role_delete(context: Context, data_dict: DataDict) -> None:
    tk.check_access("manage_user_roles", context, data_dict)

    if role := perm_model.Role.get(data_dict["id"]):
        role.delete()


@validate(schema.role_update)
def permission_role_update(context: Context, data_dict: DataDict) -> perm_types.Role:
    tk.check_access("manage_user_roles", context, data_dict)

    role = cast(perm_model.Role, perm_model.Role.get(data_dict["id"]))

    role.update(data_dict["description"])

    return role.dictize(context)


@validate(schema.permissions_update)
def permissions_update(context: Context, data_dict: DataDict) -> DataDict:
    """Update the permissions for a given permission key.

    Returns:
        A dictionary with the updated permissions and the missing permissions
    """
    tk.check_access("manage_permissions", context, data_dict)

    _validate_permission_data(data_dict)
    registered_permissions = perm_utils.get_permissions()

    updated_permissions = {}
    missing_permissions = []

    for permission_key, roles_data in data_dict["permissions"].items():
        if permission_key not in registered_permissions:
            missing_permissions.append(permission_key)
            continue

        permission_data = {}

        for role_id, flag in roles_data.items():
            role_permission = perm_model.RolePermission.get(role_id, permission_key)

            if flag and role_permission:
                continue

            if not flag and not role_permission:
                continue

            if not flag and role_permission:
                role_permission.delete()
            else:
                perm_model.RolePermission.create(role_id, permission_key)

            permission_data[role_id] = flag

        updated_permissions[permission_key] = permission_data

    model.Session.commit()

    return {
        "updated_permissions": updated_permissions,
        "missing_permissions": missing_permissions,
    }


def _validate_permission_data(data: DataDict) -> None:
    for permission_key, roles_data in data["permissions"].items():
        if not isinstance(permission_key, str):
            raise tk.ValidationError("Invalid permission key")

        if not isinstance(roles_data, dict):
            raise tk.ValidationError("Invalid permission mapping")

        for role_id, flag in roles_data.items():
            if not isinstance(flag, bool):
                raise tk.ValidationError("Invalid permission value")

            data, errors = tk.navl_validate(
                {"id": role_id},
                {
                    "id": [
                        tk.get_validator("not_empty"),
                        tk.get_validator("unicode_safe"),
                        tk.get_validator("permission_role_exists"),
                    ],
                },
            )

            if errors:
                raise tk.ValidationError(errors)



import pytest

import ckan.plugins.toolkit as tk
from ckan import model
from ckan.tests.helpers import call_action

from ckanext.permissions import const, utils
from ckanext.permissions.types import PermissionDefinition, PermissionGroup
from ckanext.permissions.utils import validate_groups


@pytest.mark.usefixtures("with_plugins")
class TestParsePermissionGroupSchemas:
    def test_valid_schema(self):
        assert utils.parse_permission_group_schemas()


@pytest.mark.usefixtures("with_plugins")
class TestParsePermissionGroupsValidation:
    def test_valid_group(self):
        validate_groups(
            {
                "new_group": PermissionGroup(
                    name="xxx",
                    description="xxx",
                    permissions=[
                        PermissionDefinition(
                            key="xxx",
                            label="xxx",
                            description="xxx",
                        )
                    ],
                )
            }
        )

    def test_group_missing_name(self):
        with pytest.raises(tk.ValidationError, match="Missing value"):
            validate_groups(
                {
                    "new_group": PermissionGroup(
                        name="",
                        description="xxx",
                        permissions=[],
                    )
                }
            )

    def test_group_missing_description(self):
        with pytest.raises(tk.ValidationError, match="Missing value"):
            validate_groups(
                {
                    "new_group": PermissionGroup(
                        name="xxx",
                        description="",
                        permissions=[],
                    )
                }
            )

    def test_group_missing_permissions(self):
        with pytest.raises(tk.ValidationError, match="Missing permissions"):
            validate_groups(
                {
                    "new_group": PermissionGroup(
                        name="xxx",
                        description="xxx",
                        permissions=[],
                    )
                }
            )

    def test_group_permissions_empty_list(self):
        with pytest.raises(tk.ValidationError, match="Missing permissions"):
            validate_groups(
                {
                    "new_group": PermissionGroup(
                        name="xxx",
                        description="xxx",
                        permissions=[],
                    )
                }
            )

    def test_missing_permission_key(self):
        with pytest.raises(tk.ValidationError, match="Missing value"):
            validate_groups(
                {
                    "new_group": PermissionGroup(
                        name="xxx",
                        description="xxx",
                        permissions=[
                            PermissionDefinition(
                                key="",
                                label="xxx",
                                description="xxx",
                            )
                        ],
                    )
                }
            )

    def test_missing_permission_label(self):
        with pytest.raises(tk.ValidationError, match="Missing value"):
            validate_groups(
                {
                    "new_group": PermissionGroup(
                        name="xxx",
                        description="xxx",
                        permissions=[
                            PermissionDefinition(key="xxx", label="", description="xxx")
                        ],
                    )
                }
            )

    def test_allow_empty_permission_description(self):
        validate_groups(
            {
                "new_group": PermissionGroup(
                    name="xxx",
                    description="xxx",
                    permissions=[
                        PermissionDefinition(key="xxx", label="xxx", description="")
                    ],
                ),
            }
        )


@pytest.mark.usefixtures("with_plugins")
class TestLoadSchemas:
    def test_valid_schemas(self):
        assert utils._load_schemas(
            [
                "ckanext.permissions:tests/data/test_group.yaml",
                "ckanext.permissions:default_group.yaml",
            ],
            "name",
        )

    def test_nonexistent_file(self):
        result = utils._load_schemas(["ckanext.permissions:tests:missing.yaml"], "name")
        assert result == {}


@pytest.mark.usefixtures("with_plugins")
class TestLoadSchema:
    def test_valid_schema(self):
        assert utils._load_schema(
            "ckanext.permissions:tests/data/test_group.yaml",
        )

    def test_nonexistent_file(self):
        assert not utils._load_schema(
            "ckanext.permissions:tests/data/missing.yaml",
        )


@pytest.mark.usefixtures("with_plugins")
class TestGetPermissionGroups:
    def test_get_permission_groups(self):
        result = utils.get_permission_groups()
        assert isinstance(result, list)


@pytest.mark.usefixtures("with_plugins")
class TestGetPermissions:
    def test_get_permissions(self):
        result = utils.get_permissions()

        assert isinstance(result, dict)
        assert result["perm_1"] == PermissionDefinition(
            key="perm_1", label="Permission 1"
        )


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestCheckPermission:
    def test_set_permission(self):
        anon_user = model.AnonymousUser()
        assert not utils.check_permission("perm_1", anon_user)

        call_action(
            "permissions_update",
            permissions={"perm_1": {const.Roles.Anonymous.value: True}},
        )

        assert utils.check_permission("perm_1", anon_user)


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestEnsureDefaultRoles:
    def test_creates_default_roles(self, reset_db, migrate_db_for):
        """Test that ensure_default_roles creates all default roles"""
        from ckanext.permissions import model as perm_model

        for role in perm_model.Role.all():
            perm_model.Role.delete(perm_model.Role.get(role["id"]))

        assert len(perm_model.Role.all()) == 0

        created_count = utils.ensure_default_roles()

        assert created_count == 3

        assert perm_model.Role.get("anonymous") is not None
        assert perm_model.Role.get("authenticated") is not None
        assert perm_model.Role.get("administrator") is not None

    def test_reuse(self):
        """Test that ensure_default_roles can be called multiple times"""
        # Call ensure_default_roles after initial call in conftest.py
        addiitonal_call = utils.ensure_default_roles()
        assert addiitonal_call == 0


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestAssignRoleToUser:
    def test_assign_role_to_user(self, user_factory):
        """Test assigning a role to a user"""
        from ckanext.permissions import model as perm_model

        user = user_factory()

        utils.assign_role_to_user(user["id"], const.Roles.Administrator.value)

        user_roles = perm_model.UserRole.get(user["id"])
        role_ids = [role.role_id for role in user_roles]
        assert const.Roles.Administrator.value in role_ids

    def test_assign_duplicate_role(self, user_factory):
        """Test that assigning the same role twice doesn't create duplicates"""
        from ckanext.permissions import model as perm_model

        user = user_factory()

        utils.assign_role_to_user(user["id"], const.Roles.Administrator.value)
        utils.assign_role_to_user(user["id"], const.Roles.Administrator.value)

        user_roles = perm_model.UserRole.get(user["id"])
        role_ids = [role.role_id for role in user_roles]
        assert len(role_ids) == 2


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestRemoveRoleFromUser:
    def test_remove_role_from_user(self, user_factory):
        """Test removing a role from a user"""
        from ckanext.permissions import model as perm_model

        user = user_factory()

        utils.assign_role_to_user(user["id"], const.Roles.Administrator.value)
        utils.remove_role_from_user(user["id"], const.Roles.Administrator.value)

        user_roles = perm_model.UserRole.get(user["id"])
        role_ids = [role.role_id for role in user_roles]
        assert const.Roles.Administrator.value not in role_ids

    def test_remove_role_doesnt_affect_other_roles(self, user_factory, test_role):
        """Test that removing one role doesn't affect other roles"""
        from ckanext.permissions import model as perm_model

        user = user_factory()

        utils.assign_role_to_user(user["id"], const.Roles.Administrator.value)

        utils.remove_role_from_user(user["id"], const.Roles.Administrator.value)

        # Verify only the correct role was removed
        user_roles = perm_model.UserRole.get(user["id"])
        role_ids = [role.role_id for role in user_roles]
        assert const.Roles.Administrator.value not in role_ids
        assert const.Roles.Authenticated.value in role_ids



from __future__ import annotations

import ckan.plugins.toolkit as tk
from ckan import model, types

import ckanext.permissions.utils as perm_utils


@tk.chained_auth_function
@tk.auth_allow_anonymous_access
def package_show(
    next_: types.AuthFunction,
    context: types.Context,
    data_dict: types.DataDict | None,
) -> types.AuthResult:
    user = model.User.get(context.get("user")) or model.AnonymousUser()
    package = context.get("package")  # type: ignore

    if not package:
        return next_(context, data_dict or {})

    # Check permissions in order of precedence
    permission_checks = [
        ("read_any_dataset", None),
        ("read_private_dataset", lambda: package.private),
    ]

    for permission, condition in permission_checks:
        if condition is not None and not condition():
            continue

        if perm_utils.check_permission(permission, user):
            return {"success": True}

    return next_(context, data_dict or {})


@tk.chained_auth_function
@tk.auth_allow_anonymous_access
def package_update(
    next_: types.AuthFunction, context: types.Context, data_dict: types.DataDict | None
) -> types.AuthResult:
    user = model.User.get(context.get("user")) or model.AnonymousUser()

    if perm_utils.check_permission("update_any_dataset", user):
        return {"success": True}

    return next_(context, data_dict or {})


@tk.chained_auth_function
@tk.auth_allow_anonymous_access
def package_delete(
    next_: types.AuthFunction, context: types.Context, data_dict: types.DataDict | None
) -> types.AuthResult:
    user = model.User.get(context.get("user")) or model.AnonymousUser()

    if perm_utils.check_permission("delete_any_dataset", user):
        return {"success": True}

    return next_(context, data_dict or {})


@tk.chained_auth_function
@tk.auth_allow_anonymous_access
def resource_delete(
    next_: types.AuthFunction, context: types.Context, data_dict: types.DataDict | None
) -> types.AuthResult:
    user = model.User.get(context.get("user")) or model.AnonymousUser()

    if perm_utils.check_permission("delete_any_resource", user):
        return {"success": True}

    return next_(context, data_dict or {})


def manage_user_roles(context: types.Context, data_dict: types.DataDict) -> types.AuthResult:
    return {"success": False}


def manage_permissions(context: types.Context, data_dict: types.DataDict) -> types.AuthResult:
    return {"success": False}



from typing import Any, Callable

import pytest

import ckan.plugins.toolkit as tk
from ckan.tests.helpers import call_action


@pytest.mark.usefixtures("with_plugins", "clean_index", "clean_db")
class TestPermissionLabels:
    def test_private_dataset_is_not_visible_to_anonymous_user(
        self, app, dataset_factory: Callable[..., dict[str, Any]]
    ):
        dataset = dataset_factory(private=True)

        app.get(tk.h.url_for("dataset.read", id=dataset["id"]), headers={}, status=404)

    def test_private_dataset_make_anon_user_to_read_private_dataset(
        self, app, dataset_factory: Callable[..., dict[str, Any]]
    ):
        dataset = dataset_factory(private=True)

        call_action(
            "permissions_update",
            permissions={"read_private_dataset": {"anonymous": True}},
        )

        app.get(tk.h.url_for("dataset.read", id=dataset["id"]), headers={}, status=200)



from __future__ import annotations

import ckan.plugins as p
import ckan.plugins.toolkit as tk
from ckan import types

from ckanext.permissions import const as perm_const
from ckanext.permissions import implementation
from ckanext.permissions import types as perm_types
from ckanext.permissions import utils


@tk.blanket.cli
@tk.blanket.validators
@tk.blanket.actions
@tk.blanket.helpers
@tk.blanket.auth_functions
@tk.blanket.config_declarations
class PermissionsPlugin(implementation.PermissionLabels, p.SingletonPlugin):
    p.implements(p.IConfigurer)
    p.implements(p.ISignal)

    _permissions_groups: perm_types.PermissionGroup | None = None
    _permissions = dict[str, perm_types.PermissionDefinition]

    # IConfigurer

    def update_config(self, config_: tk.CKANConfig):
        if not PermissionsPlugin._permissions_groups:  # type: ignore
            PermissionsPlugin._permissions_groups = list(  # type: ignore
                utils.parse_permission_group_schemas().values()
            )
            PermissionsPlugin._permissions = {  # type: ignore
                permission["key"]: permission
                for group in PermissionsPlugin._permissions_groups  # type: ignore
                for permission in group["permissions"]
            }

        tk.add_template_directory(config_, "templates")

    # ISignal

    def get_signal_subscriptions(self) -> types.SignalMapping:
        return {tk.signals.action_succeeded: [self.assign_default_user_role]}

    @staticmethod
    def assign_default_user_role(
        action_name: str,
        context: types.Context,
        data_dict: types.DataDict,
        result: types.DataDict,
    ):
        """Assign the default user role to a new user

        Args:
            action_name: The name of the action
            context: The action context
            data_dict: The action payload
            result: The action result
        """

        if action_name != "user_create":
            return

        utils.assign_role_to_user(
            result["id"], perm_const.Roles.Authenticated.value, "global"
        )



from __future__ import annotations

from ckanext.permissions import const, model, utils


def get_registered_roles() -> dict[str, str]:
    """Get the registered roles

    Returns:
        The registered roles
    """
    return utils.get_registered_roles()


def get_role_permissions(role_id: str, permission: str) -> bool:
    """Check if a role has a permission

    Args:
        role_id (str): The id of the role
        permission (str): The permission to check

    Returns:
        True if the role has the permission, False otherwise
    """
    return model.RolePermission.get(role_id, permission) is not None


def get_user_roles(user_id: str, scope: str = "global", scope_id: str | None = None) -> list[str]:
    """Get the roles of a user

    Args:
        user_id (str): The id of the user
        scope (str): The scope of the role
        scope_id (str | None): The id of the scope

    Returns:
        The roles of the user
    """
    return [str(role.role_id) for role in model.UserRole.get(user_id, scope, scope_id)]


def is_default_role(role_id: str) -> bool:
    """Check if the role is a default role

    Args:
        role_id (str): The id of the role to check

    Returns:
        True if the role is a default role, False otherwise
    """
    return any(role_id == role.value for role in const.Roles)



from __future__ import annotations

from typing import Optional, TypedDict


class PermissionGroup(TypedDict):
    name: str
    permissions: list["PermissionDefinition"]
    description: Optional[str]


class PermissionDefinition(TypedDict, total=False):
    key: str
    label: str
    description: Optional[str]


class PermissionRoleDefinition(TypedDict):
    role: str
    state: str


class PermissionRolePayload(PermissionRoleDefinition):
    permission: str


class PermissionRole(PermissionRolePayload):
    id: str


class Role(TypedDict):
    id: str
    label: str
    description: str



import pytest

from ckan.tests.helpers import call_action

from ckanext.permissions import const, helpers, model


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestGetRegisteredRoles:
    def test_get_default_roles(self):
        result = helpers.get_registered_roles()

        assert len(result) == 3

        assert result[const.Roles.Anonymous.value]
        assert result[const.Roles.Authenticated.value]
        assert result[const.Roles.Administrator.value]

    def test_with_custom_roles(self, test_role: dict[str, str]):
        result = helpers.get_registered_roles()

        assert len(result) == 4
        assert result[test_role["id"]] == test_role["label"]


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestGetRolePermissions:
    def test_no_permission(self, test_role: dict[str, str]):
        assert not helpers.get_role_permissions(test_role["id"], "read_any_dataset")

    def test_with_permission(self, test_role: dict[str, str]):
        call_action(
            "permissions_update",
            permissions={"read_any_dataset": {test_role["id"]: True}},
        )

        assert helpers.get_role_permissions(test_role["id"], "read_any_dataset")


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestGetUserRoles:
    def test_only_default_roles(self, user: dict[str, str]):
        assert helpers.get_user_roles(user["id"]) == ["authenticated"]

    def test_add_new_role(self, user: dict[str, str], test_role: dict[str, str]):
        model.UserRole.create(user["id"], test_role["id"])
        result = helpers.get_user_roles(user["id"])
        assert "authenticated" in result
        assert test_role["id"] in result

    def test_remove_role(self, user: dict[str, str], test_role: dict[str, str]):
        model.UserRole.create(user["id"], test_role["id"])
        result = helpers.get_user_roles(user["id"])
        assert "authenticated" in result
        assert test_role["id"] in result

        model.UserRole.delete(user["id"], test_role["id"])
        assert helpers.get_user_roles(user["id"]) == ["authenticated"]


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestIsDefaultRole:
    def test_is_default_role(self):
        assert helpers.is_default_role(const.Roles.Authenticated.value)

    def test_is_not_default_role(self, test_role: dict[str, str]):
        assert not helpers.is_default_role("test_role")



from typing import Any

import pytest

import ckan.plugins.toolkit as tk
from ckan.tests.helpers import call_action


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestPermissionRoleCreate:
    def test_permission_role_create(self):
        result = call_action(
            "permission_role_create",
            id="admin",
            label="Admin",
            description="Admin role",
        )
        assert result["id"] == "admin"
        assert result["label"] == "Admin"
        assert result["description"] == "Admin role"

    @pytest.mark.parametrize("field", ["id", "label", "description"])
    def test_permission_role_create_missing_required_fields(self, field):
        data = {"id": "admin", "label": "Admin", "description": "Admin role"}
        data.pop(field)

        with pytest.raises(tk.ValidationError) as e:
            call_action("permission_role_create", **data)

        assert e.value.error_dict[field] == ["Missing value"]

    def test_permission_role_create_duplicate_id(self):
        call_action(
            "permission_role_create",
            id="admin",
            label="Admin",
            description="Admin role",
        )

        with pytest.raises(tk.ValidationError) as e:
            call_action(
                "permission_role_create",
                id="admin",
                label="Another Admin",
                description="Another admin role",
            )

        assert e.value.error_dict["id"] == ["Role admin is already exists"]


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestPermissionRoleDelete:
    def test_permission_role_delete(self):
        # Create a role first
        call_action(
            "permission_role_create",
            id="custom_role",
            label="Custom Role",
            description="Custom role for testing",
        )

        call_action("permission_role_delete", id="custom_role")

        with pytest.raises(tk.ValidationError) as e:
            call_action("permission_role_delete", id="xxx")

        assert e.value.error_dict["id"] == ["Role xxx doesn't exists"]

    def test_permission_role_delete_missing_id(self):
        with pytest.raises(tk.ValidationError) as e:
            call_action("permission_role_delete")

        assert e.value.error_dict["id"] == ["Missing value"]

    def test_permission_role_delete_nonexistent_role(self):
        with pytest.raises(tk.ValidationError) as e:
            call_action("permission_role_delete", id="xxx")

        assert e.value.error_dict["id"] == ["Role xxx doesn't exists"]


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestPermissionRoleUpdate:
    def test_permission_role_update(self, test_role: dict[str, Any]):
        result = call_action("permission_role_update", id=test_role["id"], description="New description")

        assert result["id"] == test_role["id"]
        assert result["description"] == "New description"

    def test_permission_role_update_missing_id(self):
        with pytest.raises(tk.ValidationError) as e:
            call_action("permission_role_update")

        assert e.value.error_dict["id"] == ["Missing value"]

    def test_permission_role_update_nonexistent_role(self):
        with pytest.raises(tk.ValidationError) as e:
            call_action("permission_role_update", id="xxx")

        assert e.value.error_dict["id"] == ["Role xxx doesn't exists"]

    def test_permission_role_update_cant_update_label(self, test_role: dict[str, Any]):
        result = call_action(
            "permission_role_update",
            id=test_role["id"],
            label="XXX",
            description="New description",
        )

        assert test_role["label"] == result["label"]



from ckan.lib import munge

from ckanext.permissions.utils import get_registered_roles


def permission_munge_string(value: str) -> str:
    """Munge a string using CKAN's munge_name function.

    Args:
        value: The string to munge

    Returns:
        The munged string
    """
    return munge.munge_name(value)


def permission_get_registered_roles_options() -> list[dict[str, str]]:
    return [
        {"value": role_id, "text": role_label}
        for role_id, role_label in get_registered_roles().items()
        if role_id not in ("anonymous")
    ]



from __future__ import annotations

import logging
from typing import Any, Union

from flask import Blueprint, Response
from flask.views import MethodView

import ckan.model as model
import ckan.plugins.toolkit as tk
import ckan.types as types
from ckan.lib.helpers import Page

from ckanext.permissions import model as perm_model
from ckanext.permissions import utils

log = logging.getLogger(__name__)
perm_manager = Blueprint("perm_manager", __name__, url_prefix="/permissions")


USER_ROLES_PER_PAGE = 10


@perm_manager.before_request
def before_request() -> None:
    try:
        tk.check_access("sysadmin", {"user": tk.current_user.name})
    except tk.NotAuthorized:
        tk.abort(403, tk._("Need to be system administrator to administer"))


class PermissionManagerView(MethodView):
    def get(self) -> Union[str, Response]:
        return tk.render(
            "perm_manager/list.html",
            extra_vars={
                "permission_groups": utils.get_permission_groups(),
            },
        )

    def post(self) -> Response:
        try:
            tk.get_action("permissions_update")({}, {"permissions": self._get_permissions()})
        except tk.ValidationError as e:
            tk.h.flash_error(str(e))
            return tk.redirect_to("perm_manager.permission_list")

        tk.h.flash_success("Permissions updated")

        return tk.redirect_to("perm_manager.permission_list")

    def _get_permissions(self) -> dict[str, dict[str, bool]]:
        permissions = {}

        for key in tk.request.form.keys():
            if "|" not in key:
                continue

            values = tk.request.form.getlist(key)
            permission, role_id = key.split("|")

            if permission not in permissions:
                permissions[permission] = {}

            permissions[permission][role_id] = "set" in values

        return permissions


class RoleManagerView(MethodView):
    def get(self) -> Union[str, Response]:
        return tk.render(
            "perm_manager/role_list.html",
            extra_vars={
                "roles": sorted(perm_model.Role.all(), key=lambda x: x["label"]),
            },
        )


class RoleAdd(MethodView):
    def get(self) -> Union[str, Response]:
        return tk.render(
            "perm_manager/add_role.html",
            extra_vars={"errors": {}, "data": {}},
        )

    def post(self) -> Union[str, Response]:
        payload = dict(tk.request.form)

        tk.get_or_bust(payload, ["id", "label", "description"])

        try:
            tk.get_action("permission_role_create")({}, payload)
        except tk.NotAuthorized as e:
            return tk.abort(403, str(e))
        except tk.ValidationError as e:
            return tk.render(
                "perm_manager/add_role.html",
                extra_vars={"errors": e.error_dict, "data": payload},
            )

        tk.h.flash_success("Role has been created")

        return tk.redirect_to("perm_manager.role_list")


class RoleDelete(MethodView):
    def post(self) -> Response:
        payload = dict(tk.request.form)

        tk.get_or_bust(payload, ["id"])

        try:
            tk.get_action("permission_role_delete")({}, payload)
        except tk.ValidationError as e:
            tk.h.flash_error(str(e))
        else:
            tk.h.flash_success("Role has been deleted")

        return tk.redirect_to("perm_manager.role_list")


class RoleEdit(MethodView):
    def get(self, role_id: str) -> Union[str, Response]:
        return tk.render(
            "perm_manager/edit_role.html",
            extra_vars={"role": perm_model.Role.get(role_id), "errors": {}, "data": {}},
        )

    def post(self, role_id: str) -> Union[str, Response]:
        payload = dict(tk.request.form)

        tk.get_or_bust(payload, "description")

        try:
            tk.get_action("permission_role_update")(
                {},
                {
                    "id": role_id,
                    "description": payload["description"],
                },
            )
        except tk.ValidationError as e:
            return tk.render(
                "perm_manager/edit_role.html",
                extra_vars={
                    "role": perm_model.Role.get(role_id),
                    "errors": e.error_dict,
                    "data": payload,
                },
            )

        tk.h.flash_success("Role has been updated")

        return tk.redirect_to("perm_manager.role_list")


class BaseUserRolesList(MethodView):
    def _get_user_with_roles(self, scope: str = "global", scope_id: str | None = None) -> list[dict[str, Any]]:
        """
        Get all active users and their roles.
        """
        users = self._get_active_users()
        result: list[dict[str, Any]] = []

        for user in users:
            result.append(
                {
                    "id": user.id,
                    "display_name": user.display_name,
                    "roles": tk.h.get_user_roles(user.id, scope, scope_id),
                }
            )

        return self._apply_filters(result)

    def _get_active_users(self) -> list[model.User]:
        """
        Get all active users and sort them by display name.
        """
        return sorted(
            (model.Session.query(model.User).filter(model.User.state == model.State.ACTIVE).all()),
            key=lambda x: x.display_name,
        )

    def _apply_filters(self, users: list[dict[str, Any]]) -> list[dict[str, Any]]:
        """
        Apply filters based on username and role.
        """
        q = tk.request.args.get("q", "").strip()
        role_filter = tk.request.args.get("role", "").strip()

        if q:
            users = [user for user in users if q.lower() in user["display_name"].lower() or q.lower() in user["roles"]]

        if role_filter:
            users = [user for user in users if role_filter in user["roles"]]

        return users


class UserRolesList(BaseUserRolesList):
    def get(self) -> Union[str, Response]:
        users = self._get_user_with_roles()
        page = Page(
            collection=users,
            page=tk.h.get_page_number(tk.request.args),
            url=tk.h.pager_url,
            item_count=len(users),
            items_per_page=int(tk.request.args.get("limit", USER_ROLES_PER_PAGE)),
        )
        return tk.render("perm_manager/user_roles_list.html", extra_vars={"page": page})


class OrganizationUserRolesList(BaseUserRolesList):
    def get(self, org_id: str) -> Union[str, Response]:
        users = self._get_user_with_roles(scope="organization", scope_id=org_id)
        org_dict = _get_org_dict(org_id)

        def _pager_url(**kwargs):
            return tk.h.url_for("perm_manager.organization_user_roles_list", org_id=org_id, **kwargs)

        page = Page(
            collection=users,
            page=tk.h.get_page_number(tk.request.args),
            url=_pager_url,
            item_count=len(users),
            items_per_page=int(tk.request.args.get("limit", USER_ROLES_PER_PAGE)),
        )

        return tk.render(
            "perm_manager/organization/user_roles_list.html",
            extra_vars={
                "group_dict": org_dict,
                "group_type": "organization",
                "page": page,
            },
        )


class EditUserRole(MethodView):
    def __init__(self):
        self.schema = {
            "roles": [tk.get_validator(validator) for validator in "not_missing list_of_strings roles_exists".split()]
        }

    def get(self, user_id: str) -> Union[str, Response]:
        user = model.User.get(user_id)

        if not user:
            return tk.abort(404, "User not found")

        return tk.render(
            "perm_manager/edit_user_roles.html",
            extra_vars={
                "user": user,
                "data": {"roles": ",".join(tk.h.get_user_roles(user.id))},
                "errors": {},
            },
        )

    def post(self, user_id: str) -> Union[str, Response]:
        return self._update_user_roles(user_id, "global")

    def _update_user_roles(self, user_id: str, scope: str, scope_id: str | None = None) -> Union[str, Response]:
        payload = {"roles": tk.request.form.getlist("roles")}

        user = model.User.get(user_id)

        if not user:
            tk.abort(404, "User not found")

        data, errors = tk.navl_validate(payload, self.schema)

        if errors:
            return tk.render(
                "perm_manager/edit_user_roles.html",
                extra_vars={"user": user, "data": data, "errors": errors},
            )

        perm_model.UserRole.clear_user_roles(user.id, scope, scope_id)

        for role in data["roles"]:
            perm_model.UserRole.create(user_id=user.id, role=role, scope=scope, scope_id=scope_id)

        model.Session.commit()

        tk.h.flash_success("User roles updated")

        return (
            tk.redirect_to("perm_manager.user_roles_list")
            if scope == "global"
            else tk.redirect_to("perm_manager.organization_user_roles_list", org_id=scope_id)
        )


class OrganizationEditUserRole(EditUserRole):
    def get(self, org_id: str, user_id: str) -> Union[str, Response]:
        user = model.User.get(user_id)

        if not user:
            return tk.abort(404, "User not found")

        org_dict = _get_org_dict(org_id)
        scope = "organization"

        return tk.render(
            "perm_manager/organization/edit_user_roles.html",
            extra_vars={
                "user": user,
                "data": {"roles": ",".join(tk.h.get_user_roles(user.id, scope, org_dict["id"]))},
                "errors": {},
                "group_dict": org_dict,
                "group_type": scope,
            },
        )

    def post(self, org_id: str, user_id: str) -> Union[str, Response]:
        return self._update_user_roles(user_id, "organization", org_id)


def _get_org_dict(org_id: str) -> dict[str, Any]:
    context = types.Context(user=tk.current_user.name, for_view=True)

    try:
        return tk.get_action("organization_show")(context, {"id": org_id, "include_datasets": False})
    except (tk.ObjectNotFound, tk.NotAuthorized):
        tk.abort(404, tk._("Organization not found"))


perm_manager.add_url_rule("/manage", view_func=PermissionManagerView.as_view("permission_list"))

perm_manager.add_url_rule("/roles", view_func=RoleManagerView.as_view("role_list"))
perm_manager.add_url_rule("/roles/add", view_func=RoleAdd.as_view("role_add"))
perm_manager.add_url_rule("/roles/delete", view_func=RoleDelete.as_view("role_delete"))
perm_manager.add_url_rule("/roles/<role_id>", view_func=RoleEdit.as_view("role_edit"))

# organization user roles
perm_manager.add_url_rule(
    "/organization/user-roles/<org_id>",
    view_func=OrganizationUserRolesList.as_view("organization_user_roles_list"),
)
perm_manager.add_url_rule(
    "/organization/user-roles/<org_id>/<user_id>",
    view_func=OrganizationEditUserRole.as_view("organization_edit_user_role"),
)

perm_manager.add_url_rule("/user-roles", view_func=UserRolesList.as_view("user_roles_list"))
perm_manager.add_url_rule("/user-roles/<user_id>", view_func=EditUserRole.as_view("edit_user_role"))

blueprints = [perm_manager]



from __future__ import annotations

from typing import Callable

import pytest

import ckan.plugins.toolkit as tk

import ckanext.permissions.logic.validators as perm_validators
from ckanext.permissions.const import ROLE_ID_MAX_LENGTH, Roles


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestRoleDoesntExists:
    def test_role_doesnt_exists_valid(self):
        """Test role_doesnt_exists with non-existent role."""
        assert perm_validators.role_doesnt_exists("new_role")

    def test_role_doesnt_exists_invalid(self, test_role: dict[str, str]):
        """Test role_doesnt_exists with existing role."""
        with pytest.raises(tk.Invalid) as e:
            perm_validators.role_doesnt_exists(test_role["id"])

        assert e.value.error == f"Role {test_role['id']} is already exists"


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestRoleExists:
    def test_role_exists_valid(self, test_role: dict[str, str]):
        """Test permission_role_exists with existing role."""
        assert (
            perm_validators.permission_role_exists(test_role["id"]) == test_role["id"]
        )

    def test_role_exists_invalid(self):
        """Test permission_role_exists with non-existent role."""
        role_name = "non_existent_role"
        with pytest.raises(tk.Invalid) as e:
            perm_validators.permission_role_exists(role_name)

        assert e.value.error == f"Role {role_name} doesn't exists"


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestRolesExists:
    def test_roles_exists_valid(self, role_factory: Callable[..., dict[str, str]]):
        """Test roles_exists with existing roles."""
        role_names = ["creator", "moderator"]
        for role in role_names:
            role_factory(id=role)

        assert perm_validators.roles_exists(role_names) == role_names

    def test_roles_exists_invalid(self, role_factory: Callable[..., dict[str, str]]):
        """Test roles_exists with non-existent role."""
        role_names = ["creator", "moderator"]
        role_factory(id=role_names[0])

        with pytest.raises(tk.Invalid) as e:
            perm_validators.roles_exists(role_names)

        assert e.value.error == f"Role {role_names[1]} doesn't exists"


class TestRoleIdValidator:
    def test_valid_role_id(self):
        """Test role_id_validator with valid role ID."""
        role_id = "valid-role-id"
        assert perm_validators.role_id_validator(role_id) == role_id

    @pytest.mark.parametrize(
        "invalid_id",
        [
            "A",  # too short
            "a" * (ROLE_ID_MAX_LENGTH + 1),  # too long
            "Invalid",  # uppercase
            "invalid@role",  # invalid character
            "test1",  # invalid character
        ],
    )
    def test_invalid_role_id(self, invalid_id: str):
        """Test role_id_validator with invalid role IDs."""
        with pytest.raises(tk.Invalid):
            perm_validators.role_id_validator(invalid_id)


class TestNotDefaultRole:
    def test_valid_custom_role(self):
        """Test not_default_role with custom role."""
        role_id = "custom-role"
        assert perm_validators.not_default_role(role_id) == role_id

    def test_invalid_default_role(self):
        """Test not_default_role with default role."""
        with pytest.raises(tk.Invalid) as e:
            perm_validators.not_default_role(Roles.Administrator.value)

        assert e.value.error == f"Role {Roles.Administrator.value} is a default role."



from __future__ import annotations

import ckan.plugins as p
from ckan import model
from ckan.lib.plugins import DefaultPermissionLabels

import ckanext.permissions.utils as perm_utils


class PermissionLabels(p.SingletonPlugin, DefaultPermissionLabels):
    p.implements(p.IPermissionLabels)

    def get_dataset_labels(self, dataset_obj: model.Package) -> list[str]:
        labels: list[str] = super().get_dataset_labels(dataset_obj)

        labels.append("permission-allowed")

        if dataset_obj.owner_org:
            labels.append(f"permission-allowed-org:{dataset_obj.owner_org}")

        return labels

    def get_user_dataset_labels(self, user_obj: model.User | None) -> list[str]:
        """Get permission labels for a user.

        Extends the default CKAN permission labels by checking if the user has
        special read permissions (read_any_dataset or read_private_dataset).
        If they do, adds a 'permission-allowed' label that grants access to
        the dataset.

        Args:
            user_obj: The user to get labels for. Can be None for anonymous users.

        Returns:
            List of permission labels for this user
        """
        labels: list[str] = super().get_user_dataset_labels(user_obj)  # type: ignore

        user = user_obj or model.AnonymousUser()

        for permission in ["read_any_dataset", "read_private_dataset"]:
            if perm_utils.check_permission(permission, user):
                labels.append("permission-allowed")

        return labels



from __future__ import annotations

import ckan.plugins as p
import ckan.plugins.toolkit as tk
import ckan.types as types


@tk.blanket.blueprints
@tk.blanket.helpers
class PermissionsManagerPlugin(p.SingletonPlugin):
    p.implements(p.IConfigurer)
    p.implements(p.ISignal)

    # IConfigurer

    def update_config(self, config_):
        tk.add_template_directory(config_, "templates")
        tk.add_public_directory(config_, "public")
        tk.add_resource("assets", "permissions_manager")

    # ISignal

    def get_signal_subscriptions(self) -> types.SignalMapping:
        return {
            tk.signals.ckanext.signal("ap_main:collect_config_sections"): [self.collect_config_sections_subs],
        }

    @staticmethod
    def collect_config_sections_subs(sender: None):
        return {
            "name": "Permissions",
            "configs": [
                {
                    "name": "Permissions manager",
                    "blueprint": "perm_manager.permission_list",
                    "info": "Manage portal permissions",
                },
                {
                    "name": "Roles manager",
                    "blueprint": "perm_manager.role_list",
                    "info": "Manage portal roles",
                },
                {
                    "name": "User roles list",
                    "blueprint": "perm_manager.user_roles_list",
                },
            ],
        }



import pytest

import ckan.plugins.toolkit as tk
from ckan.tests.helpers import call_action

from ckanext.permissions import model as perm_model


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestPermissionsUpdate:
    def test_permissions_update(self):
        result = call_action(
            "permissions_update",
            permissions={
                "perm_1": {
                    "anonymous": False,
                    "authenticated": True,
                    "administrator": True,
                },
                "perm_2": {
                    "anonymous": False,
                    "authenticated": True,
                    "administrator": True,
                },
            },
        )

        assert not result["missing_permissions"]
        assert result["updated_permissions"] == {
            "perm_1": {"authenticated": True, "administrator": True},
            "perm_2": {"authenticated": True, "administrator": True},
        }

        assert not perm_model.RolePermission.get("anonymous", "perm_1")
        assert perm_model.RolePermission.get("authenticated", "perm_1")
        assert perm_model.RolePermission.get("administrator", "perm_1")

        result = call_action(
            "permissions_update",
            permissions={"perm_1": {"anonymous": True}},
        )

        assert perm_model.RolePermission.get("anonymous", "perm_1")

    def test_permissions_update_unregistered_permission_key(self):
        result = call_action("permissions_update", permissions={"xxx": {}})

        assert result["missing_permissions"] == ["xxx"]
        assert not result["updated_permissions"]

    def test_not_string_permission_key(self):
        with pytest.raises(tk.ValidationError, match="Invalid permission key"):
            call_action("permissions_update", permissions={1: {}})

    def test_not_dict_roles_mapping(self):
        with pytest.raises(tk.ValidationError, match="Invalid permission mapping"):
            call_action("permissions_update", permissions={"perm_1": 1})

    def test_not_bool_permission_value(self):
        with pytest.raises(tk.ValidationError, match="Invalid permission value"):
            call_action(
                "permissions_update",
                permissions={"perm_1": {"anonymous": "xxx", "authenticated": "yyy"}},
            )

    def test_role_id_not_exists(self):
        with pytest.raises(tk.ValidationError, match="Role xxx doesn't exists"):
            call_action("permissions_update", permissions={"perm_1": {"xxx": True}})



from __future__ import annotations

from ckan import types
from ckan.logic.schema import validator_args


@validator_args
def role_create(not_empty, unicode_safe, role_id_validator, role_doesnt_exists, ignore) -> types.Schema:
    return {
        "id": [not_empty, unicode_safe, role_id_validator, role_doesnt_exists],
        "label": [not_empty, unicode_safe],
        "description": [not_empty, unicode_safe],
        "__extras": [ignore],
    }


@validator_args
def role_delete(not_empty, unicode_safe, permission_role_exists, not_default_role) -> types.Schema:
    return {
        "id": [not_empty, unicode_safe, permission_role_exists, not_default_role],
    }


@validator_args
def role_update(not_empty, unicode_safe, permission_role_exists) -> types.Schema:
    return {
        "id": [not_empty, unicode_safe, permission_role_exists],
        "description": [not_empty, unicode_safe],
    }


@validator_args
def permissions_update(default, convert_to_json_if_string, dict_only) -> types.Schema:
    return {
        "permissions": [default("{}"), convert_to_json_if_string, dict_only],
    }


@validator_args
def permission_group_schema(not_empty, unicode_safe) -> types.Schema:
    return {
        "name": [not_empty, unicode_safe],
        "description": [not_empty, unicode_safe],
        "permissions": permission_schema(),
    }


@validator_args
def permission_schema(not_empty, unicode_safe, ignore_missing) -> types.Schema:
    return {
        "key": [not_empty, unicode_safe],
        "label": [not_empty, unicode_safe],
        "description": [ignore_missing, unicode_safe],
    }



from __future__ import annotations

import inspect
import os
from typing import cast

import yaml

import ckan.plugins.toolkit as tk
from ckan import model

import ckanext.permissions.const as perm_const
import ckanext.permissions.logic.schema as perm_schema
import ckanext.permissions.model as perm_model
import ckanext.permissions.types as perm_types


def parse_permission_group_schemas() -> dict[str, perm_types.PermissionGroup]:
    groups = _load_schemas(
        tk.aslist(tk.config.get("ckanext.permissions.permission_groups")), "name"
    )

    validate_groups(groups)

    return groups


def _load_schemas(schemas: list[str], type_field: str):
    result = {}

    for path in schemas:
        schema = _load_schema(path)

        if not schema:
            continue

        result[schema[type_field]] = schema

    return result


def _load_schema(path: str):
    """
    Given a path like "ckanext.permissions:default_group.yaml"
    find the second part relative to the import path of the first
    """

    module, file_name = path.split(":", 1)

    try:
        imp_module = __import__(module, fromlist=[""])
    except ImportError:
        return

    file_path = os.path.join(os.path.dirname(inspect.getfile(imp_module)), file_name)

    if not os.path.exists(file_path):
        return

    with open(file_path) as file:
        return yaml.safe_load(file)


def validate_groups(groups: dict[str, perm_types.PermissionGroup]) -> bool:
    permissions = []

    for group in groups.values():
        data, errors = tk.navl_validate(
            cast(dict, group), perm_schema.permission_group_schema()
        )

        if errors:
            raise tk.ValidationError(errors)

        if not data.get("permissions"):
            raise tk.ValidationError("Missing permissions")

        if not isinstance(data["permissions"], list):
            raise tk.ValidationError("Permissions must be a list")

        for permission in data["permissions"]:
            if permission["key"] in permissions:
                raise tk.ValidationError(
                    f"Permission {permission['key']} is duplicated"
                )

            permissions.append(permission["key"])

    return True


def get_permission_groups() -> list[perm_types.PermissionGroup]:
    from ckanext.permissions.plugin import PermissionsPlugin

    return PermissionsPlugin._permissions_groups  # type: ignore


def get_permissions() -> dict[str, perm_types.PermissionDefinition]:
    from ckanext.permissions.plugin import PermissionsPlugin

    return PermissionsPlugin._permissions  # type: ignore


def get_registered_roles() -> dict[str, str]:
    return {role["id"]: role["label"] for role in perm_model.Role.all()}


def check_permission(
    permission: str, user: model.User | model.AnonymousUser, scope: str = "global"
) -> bool:
    """Check if user has the given permission through any of their roles.

    Args:
        permission: The permission key to check
        user: The user to check permissions for

    Returns:
        bool: True if user has the permission, False otherwise
    """
    if isinstance(user, model.AnonymousUser):
        return (
            perm_model.RolePermission.get(perm_const.Roles.Anonymous.value, permission)
            is not None
        )

    for role in user.roles:  # type: ignore
        if scope not in role.scope:
            continue

        if perm_model.RolePermission.get(str(role.role_id), permission) is not None:
            return True

    return False


def assign_role_to_user(
    user_id: str, role_id: str, scope: str = "global", scope_id: str | None = None
):
    """Assign role to an User.

    Args:
        role_id: The role to assign
        user_id: The user to assign the role to
        scope: The scope of the role
        scope_id: The scope ID of the role
    """

    scope_roles = perm_model.UserRole.get(user_id, scope, scope_id)

    if role_id not in [role.role_id for role in scope_roles]:
        perm_model.UserRole.create(user_id, role_id)


def remove_role_from_user(user_id: str, role_id: str):
    """Remove role from an User.

    Args:
        role_id: The role to remove
        user_id: The user to remove the role from
    """

    roles = perm_model.UserRole.get(user_id)

    if role_id in [role.role_id for role in roles]:
        perm_model.UserRole.delete(user_id, role_id)


def ensure_default_roles() -> int:
    """Ensure default roles exist in the database.

    Creates anonymous, authenticated, and administrator roles if they don't exist.

    Returns:
        int: Number of roles created
    """
    default_roles = [
        ("anonymous", "Anonymous", "Default role for anonymous users"),
        (
            "authenticated",
            "Authenticated",
            "Regular user that will be assigned automatically for all users on a portal",
        ),
        (
            "administrator",
            "Administrator",
            "Administrator that should have all permissions",
        ),
    ]

    created_count = 0
    for role_id, label, description in default_roles:
        existing_role = perm_model.Role.get(role_id)
        if not existing_role:
            perm_model.Role.create(role_id, label, description)
            created_count += 1

    return created_count



import pytest

import ckan.plugins.toolkit as tk
from ckan.tests.helpers import call_action

import ckanext.permissions.model as perm_model


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestRoleAutoassignment:
    def test_assign_user_role_on_create(self, user):
        """Test if a role is assigned to a user when it's created"""
        roles = tk.h.get_user_roles(user["id"])
        assert roles == ["authenticated"]

    def test_user_roles_deleted_with_role(self, user, test_role):
        """Test if user roles are deleted when the role is deleted"""
        perm_model.UserRole.create(user["id"], test_role["id"])

        result = tk.h.get_user_roles(user["id"])
        assert "authenticated" in result
        assert "creator" in result

        call_action("permission_role_delete", id=test_role["id"])

        assert tk.h.get_user_roles(user["id"]) == ["authenticated"]



from __future__ import annotations

import re

import ckan.plugins.toolkit as tk

import ckanext.permissions.const as perm_const
import ckanext.permissions.model as perm_model


def role_doesnt_exists(role: str) -> str:
    """Ensure that a role doesn't exists.

    Args:
        role (str): role name

    Raises:
        tk.Invalid: if the role exists

    Returns:
        role name
    """
    if perm_model.Role.get(role) is not None:
        raise tk.Invalid(f"Role {role} is already exists")

    return role


def permission_role_exists(role: str) -> str:
    """Ensure that a role exists.

    Args:
        role (str): role name

    Raises:
        tk.Invalid: if the role doesn't exists

    Returns:
        role name
    """
    if perm_model.Role.get(role) is None:
        raise tk.Invalid(f"Role {role} doesn't exists")

    return role


def roles_exists(roles: list[str]) -> list[str]:
    """Ensure that all roles exists.

    Args:
        roles (list[str]): list of roles

    Raises:
        tk.Invalid: if a role doesn't exists

    Returns:
        list of roles
    """
    for role in roles:
        permission_role_exists(role)

    return roles


def role_id_validator(value: str) -> str:
    """Validate a role ID.

    Ensures that:
        - the role ID is a string
        - is at least N characters long
        - is at most N characters long
        - contains only lowercase alpha (ascii) characters and these symbols: -_

    Args:
        value (str): role ID

    Raises:
        tk.Invalid: if the role ID is invalid

    Returns:
        role ID
    """
    name_match = re.compile(r"[a-z_\-]*$")

    if len(value) < perm_const.ROLE_ID_MIN_LENGTH:
        raise tk.Invalid(f"Role ID must be at least {perm_const.ROLE_ID_MIN_LENGTH} characters long.")

    if len(value) > perm_const.ROLE_ID_MAX_LENGTH:
        raise tk.Invalid(f"Role ID must be a maximum of {perm_const.ROLE_ID_MAX_LENGTH} characters long.")

    if not name_match.match(value):
        raise tk.Invalid("Role ID must be purely lowercase alpha(ascii) characters and these symbols: -_")

    return value


def not_default_role(role_id: str) -> str:
    """Ensure that the role is not a default role.

    Args:
        role_id (str): role ID

    Raises:
        tk.Invalid: if the role is a default role

    Returns:
        role ID
    """
    if role_id in [role.value for role in perm_const.Roles]:
        raise tk.Invalid(f"Role {role_id} is a default role.")

    return role_id



from __future__ import annotations

import factory
import pytest
from faker import Faker
from pytest_factoryboy import register

from ckan.tests import factories

import ckanext.permissions.model as perm_model
from ckanext.permissions import utils as perm_utils

fake = Faker()


@pytest.fixture()
def clean_db(reset_db, migrate_db_for):
    reset_db()
    migrate_db_for("permissions")

    perm_utils.ensure_default_roles()


@register(_name="test_role")
class RoleFactory(factories.CKANFactory):
    class Meta:
        model = perm_model.Role
        action = "permission_role_create"

    id = "creator"
    label = "Creator"
    description = factory.LazyFunction(lambda: fake.sentence(nb_words=5))


@register(_name="dataset")
class DatasetFactory(factories.Dataset):
    owner_org = factory.LazyFunction(lambda: OrganizationFactory()["id"])


@register(_name="organization")
class OrganizationFactory(factories.Organization):
    pass


@register(_name="sysadmin")
class SysadminFactory(factories.SysadminWithToken):
    pass


@register(_name="user")
class UserFactory(factories.UserWithToken):
    pass



import click

import ckan.model as model

import ckanext.permissions.const as perm_const
from ckanext.permissions import model as perm_model
from ckanext.permissions import utils

__all__ = ["permissions"]


@click.group()
def permissions():
    """Permissions management commands."""


@permissions.command()
def init_default_roles():
    """Create default roles (anonymous, authenticated, administrator) in the database"""
    created_count = utils.ensure_default_roles()

    if created_count > 0:
        click.secho(f"{created_count} role(s) created successfully", fg="green")
    else:
        click.secho("All default roles already exist", fg="yellow")

    return created_count


@permissions.command()
@click.argument("role", default=perm_const.Roles.Authenticated.value, required=False)
def assign_default_user_roles(role: str):
    """Assign automatic roles to users (initializes default roles if needed)"""
    # Ensure default roles exist
    click.echo("Checking default roles...")
    created_count = utils.ensure_default_roles()

    if created_count > 0:
        click.secho(f"{created_count} role(s) created", fg="green")
        click.echo()  # Empty line for readability

    # Check if the specified role exists
    if not perm_model.Role.get(role):
        click.secho(f"Error: Role '{role}' does not exist", fg="red")
        return

    users = model.Session.query(model.User).filter(model.User.state == "active").all()

    for user in users:
        utils.assign_role_to_user(user.id, role)

    click.secho(f"Role '{role}' assigned to {len(users)} active user(s)", fg="green")


@permissions.command()
@click.argument("role", default=perm_const.Roles.Authenticated.value, required=False)
@click.option(
    "--user-ids",
    "-u",
    multiple=True,
    help="User IDs to remove role from (if not specified, removes from all users)",
)
def remove_role_from_users(role: str, user_ids: tuple[str, ...]):
    """Remove automatic roles from users"""
    if user_ids:
        users = (
            model.Session.query(model.User).filter(model.User.id.in_(user_ids)).all()
        )
    else:
        users = model.User.all()

    for user in users:
        utils.remove_role_from_user(user.id, role)

    if user_ids:
        click.secho(
            f"Role '{role}' removed from {len(users)} specified user(s)", fg="green"
        )
    else:
        click.secho(f"Role '{role}' removed from all users", fg="green")
